<?php

namespace Drupal\tapis_app_webform\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformElement\ManagedFile;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Provides a 'tapis_app_webform_managed_file' element.
 *
 * @WebformElement(
 *   id = "tapis_app_webform_managed_file",
 *   label = @Translation("File (OSP)"),
 *   description = @Translation("Provides an element for uploading files."),
 *   category = @Translation("One Science Place (OSP)"),
 * )
 *
 * @see \Drupal\webform_example_element\Element\WebformExampleElement
 * @see \Drupal\webform\Plugin\WebformElementBase
 * @see \Drupal\webform\Plugin\WebformElementInterface
 * @see \Drupal\webform\Annotation\WebformElement
 */
class OSPWebformManagedFile extends ManagedFile {

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface $webform_submission = NULL) {
    $element['#type'] = 'managed_file';
    parent::prepare($element, $webform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    // Here you can define and alter a webform element's properties UI.
    // Form element property visibility and default values are defined via
    // ::defaultProperties.
    //
    // @see \Drupal\webform\Plugin\WebformElementBase::form
    // @see \Drupal\webform\Plugin\WebformElement\TextBase::form
    // Hide the uri scheme form element.
    $form['file']['uri_scheme']['#access'] = FALSE;

    // Get element properties.
    $element_properties = $form_state->getValues() ?: $form_state->get('element_properties');

    // Add a details element to $form for the following fields:
    // - cmdPrefix, cmdSuffix.
    $form['element']['cmd'] = [
      '#type' => 'details',
      '#access' => TRUE,
      '#title' => $this->t('Command arguments'),
      '#open' => TRUE,
    ];

    $form['element']['cmd']['valueType'] = [
      '#type' => 'select',
      '#access' => TRUE,
      '#title' => $this->t('Type'),
      '#description' => $this->t('The type of this value'),
      '#options' => [
        'cli_arg' => $this->t('Command line argument'),
        'env_var' => $this->t('Environment variable'),
        'scheduler_option' => $this->t('Scheduler option'),
      ],
      '#default_value' => $element_properties['valueType'] ?? 'cli_arg',
    ];

    // Add a textfield element to $form['cmd'] for the cmdPrefix property.
    $form['element']['cmd']['cmdPrefix'] = [
      '#type' => 'textfield',
      '#access' => TRUE,
      '#title' => $this->t('Command prefix'),
      '#description' => $this->t('The prefix to be added to the command.'),
      '#default_value' => $element_properties['cmdPrefix'] ?? '',
    ];

    // Add a checkbox element to $form['cmd']
    // for the cmdPrefix_trailing_space property.
    $form['element']['cmd']['cmdPrefix_trailing_space'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Add trailing space to command prefix'),
      '#description' => $this->t('Add a trailing space to the command prefix.'),
      '#default_value' => $element_properties['cmdPrefix_trailing_space'] ?? TRUE,
    ];

    // Add a radio element to $form.
    $form['element']['cmd']['cmdValue'] = [
      '#type' => 'radios',
      '#title' => $this->t('Command value'),
      '#options' => [
        'none' => $this->t('None'),
        'file_path' => $this->t('File path'),
        'file_directory' => $this->t('File directory'),
      ],
      '#default_value' => $element_properties['cmdValue'] ?? 'file_path',
      '#description' => $this->t('Choose how the uploaded file will be represented in the command-line args passed to the app. "File path" will insert the uploaded file\'s path within the cmd-line args (only applies for single file uploads), whereas "File directory" will insert the parent directory containing the uploaded files. "None" will not insert the file path nor folder directory into the cmd-line args (but the user-uploaded file will still be available in /TapisInput, and if provided, the command prefix and suffix will be included in the cmd args).'),
    ];

    // Add a textfield element to $form['cmd'] for the cmdSuffix property.
    $form['element']['cmd']['cmdSuffix'] = [
      '#type' => 'textfield',
      '#access' => TRUE,
      '#title' => $this->t('Command suffix'),
      '#description' => $this->t('The suffix to be added to the command.'),
      '#default_value' => $element_properties['cmdSuffix'] ?? '',
    ];

    // Add a checkbox element to $form['cmd']
    // for the cmdSuffix_trailing_space property.
    $form['element']['cmd']['cmdSuffix_trailing_space'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Add trailing space to command suffix'),
      '#description' => $this->t('Add a trailing space to the command suffix.'),
      '#default_value' => $element_properties['cmdSuffix_trailing_space'] ?? TRUE,
    ];

    // Add a checkbox element to $form['cmd']
    // for whether this file argument should be passed in last to the command.
    $form['element']['cmd']['cmdLast'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Make this the last command argument'),
      '#description' => $this->t('Pass this file input as the last argument in the generated command.'),
      '#default_value' => $element_properties['cmdLast'] ?? FALSE,
    ];

    // Add a checkbox element to $form['cmd'] for the ignoreValue property.
    $form['element']['cmd']['ignoreValue'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Ignore value'),
      '#description' => $this->t('Ignore the value of this element when generating the command.'),
      '#default_value' => $element_properties['ignoreValue'] ?? FALSE,
    ];

    // Add a checkbox element to $form['cmd']
    // for the showOnlyOnRestart property.
    $form['element']['cmd']['showOnlyOnRestart'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Show only on restart'),
      '#description' => $this->t('Show this element only when the job is restarted.'),
      '#default_value' => $element_properties['showOnlyOnRestart'] ?? FALSE,
    ];

    $form['element']['cmd']['repeatCmdArgs'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Repeat command arguments'),
      '#description' => $this->t('Repeat command arguments for multiple files.'),
      '#default_value' => $element_properties['repeatCmdArgs'] ?? FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineDefaultProperties() {
    // Here you define your webform element's default properties,
    // which can be inherited.
    //
    // @see \Drupal\webform\Plugin\WebformElementBase::defaultProperties
    // @see \Drupal\webform\Plugin\WebformElementBase::defaultBaseProperties
    return [
      'valueType' => 'cli_arg',
      'cmdPrefix' => '',
      'cmdPrefix_trailing_space' => TRUE,
      'cmdSuffix' => '',
      'cmdSuffix_trailing_space' => TRUE,
      'cmdLast' => FALSE,
      // One of none, file_path, or file_directory.
      'cmdValue' => 'file_path',
      'ignoreValue' => FALSE,
      'showOnlyOnRestart' => FALSE,
      'repeatCmdArgs' => FALSE,
      'file_extensions' => 'gz jpg json png tar tgz txt zip',
    ] + parent::defineDefaultProperties();
  }

  /**
   * {@inheritdoc}
   */
  protected function defineTranslatableProperties() {
    return array_merge(
      parent::defineTranslatableProperties(),
      [
        'valueType',
        'cmdPrefix',
        'cmdPrefix_trailing_space',
        'cmdSuffix',
        'cmdSuffix_trailing_space',
        'cmdLast',
        'cmdValue',
        'ignoreValue',
        'showOnlyOnRestart',
        'repeatCmdArgs',
      ]
    );
  }

}
